@@ -4,8 +4,6 @@ BASE_UUID_FORMAT = "0000{}-0000-1000-8000-00805f9b34fb"
 # "ffd4" - Triones:B30200000459C - legacy
 
 
-POWER_OFF_COMAMND = b"\xCC\x24\x33"
-POWER_ON_COMMAND = b"\xCC\x23\x33"
 STATE_COMMAND = b"\xEF\x01\x77"
 
 
